#define _ADALLOCC_C_
#define _ADOLC_SRC_
/*
   --------------------------------------------------------------
   File adallocc.c of ADOL-C version 1.8.0        as of Nov/30/98
   --------------------------------------------------------------
   C allocation of arrays of doubles in several dimensions 

   Last changes:
        981130 olvo: newly created.
   --------------------------------------------------------------
*/

/****************************************************************************/
/*                                                                 INCLUDES */
#include "dvlparms.h" /* Developers Parameters */
#include "usrparms.h" /* Users Parameters      */
#include "adalloc.h"

#include <stdio.h>

#ifdef __cplusplus
#include <malloc.h>
extern "C" {
#endif


/****************************************************************************/
/*                                              MEMORY MANAGEMENT UTILITIES */

/*--------------------------------------------------------------------------*/
double* myalloc1(int m)
{ double* A = (double*)malloc(m*sizeof(double));
  if (A == NULL)
  { fprintf(DIAG_OUT,"ADOL-C error: myalloc1 cannot allocate %i bytes\n",
                     m*sizeof(double));
    exit (-1);
  }
  return A;
}

/*--------------------------------------------------------------------------*/
double** myalloc2(int m, int n)
{ double *Adum = (double*)malloc(m*n*sizeof(double));
  double   **A = (double**)malloc(m*sizeof(double*));
  int i;
  if (Adum == NULL)
  { fprintf(DIAG_OUT,"ADOL-C error: myalloc2 cannot allocate %i bytes\n",
                     m*n*sizeof(double));
    exit (-1);
  }
  if (A == NULL)
  { fprintf(DIAG_OUT,"ADOL-C error: myalloc2 cannot allocate %i bytes\n",
                     m*sizeof(double*));
    exit (-1);
  }
  for (i=0; i<m; i++)
  { A[i] = Adum;
    Adum += n;
  }
  return A;

  /* To deallocate an array set up by   A = myalloc2(m,n)   */
  /*   use  free((char*)*A); free((char*)A); in that order  */
}

/*--------------------------------------------------------------------------*/
double*** myalloc3(int m, int n, int p)
{ /* This function allocates 3-tensors contiguously */ 
  double *Adum = (double*) malloc(m*n*p*sizeof(double));
  double **Apt = (double**)malloc(m*n*sizeof(double*));
  double  ***A = (double***)malloc(m*sizeof(double**));
  int i,j;
  if (Adum == NULL)
  { fprintf(DIAG_OUT,"ADOL-C error: myalloc3 cannot allocate %i bytes\n",
                     m*n*p*sizeof(double));
    exit (-1);
  }
  if (Apt == NULL)
  { fprintf(DIAG_OUT,"ADOL-C error: myalloc3 cannot allocate %i bytes\n",
                     m*n*sizeof(double*));
    exit (-1);
  }
  if (A == NULL)
  { fprintf(DIAG_OUT,"ADOL-C error: myalloc3 cannot allocate %i bytes\n",
                     m*sizeof(double**));
    exit (-1);
  }
  for (i=0; i<m; i++)
  { A[i] = Apt;
    for (j=0; j<n; j++)
    { *Apt++ =  Adum;
      Adum += p;
    }
  }   
  return A;

  /* To deallocate an array set up by  A = myalloc3(m,n,p)  */
  /* use free((char*)**A); free((char*)*A); free((char*)A); */
}


/****************************************************************************/
/*                                                                THAT'S ALL*/
#ifdef __cplusplus
}
#endif

#undef _ADOLC_SRC_
#undef _ADALLOCC_C_

